/*  file:     dbhead.c
**  purpose:  program to list the records in a dbaseiii file
**  usage:    dbhead filename
**  notes:    compile with "tcc dbhead d_open d_getrec d_getfld d_close"
**  author:   Carl Goette
**            (large amounts of code taken from sample programs by Mark Sadler)
**  revised:  7/12/89
*/

#include        <stdio.h>
#include        <memory.h>
#include        <malloc.h>
#include        "dbf.h"
#define         min(x,y)        ((x)<(y) ? (x) : (y))

/* routine to format dbase date field - buff in is in form  YYYYMMDD
**                                      buff out is in form MM/DD/YY
*/
void fdate(char *buff)
{
    int day;
    int mon;
    int yr;
    if (buff[0] == ' ')
    {
	buff[0]='\0';
	strcat(buff,"  /  /  ");
    }
    else
    {
	sscanf(buff,"%4d%02d%02d",&yr,&mon,&day);
	sprintf(buff,"%02d/%02d/%02d",mon,day,yr-1900);
    }
}


main(int argc,char **argv)
{
    int errornum;
    long rec;
    int fld;
    int listfields ;
    int listrecords = 15 ;
    struct DBF *d;
    static char buff[255];

    if(argc == 1)
    {
	printf("Usage  : dbhead [options] filename\n"
	    "Options: -r# where # is the number of records to list (default 15)\n"
	    "         -f# where # is the number of fields to list (default all)\n");
	exit(1);
    }

    /* Process arguments */
    while((*++argv))
    {
	if(*argv[0] == '/' || *argv[0] == '-') /* Process options */
	{
	    switch( (*argv)[1] )
	    {
	    case 'r': 
	    case 'R':
		listrecords = atoi(&(*argv)[2]) ;
		continue ;
	    case 'f': 
	    case 'F':
		listfields = atoi(&(*argv)[2]) ;
		continue ;
	    default:
		printf("\nInvalid option: -%cn\f",*argv[1]);
		break ;
	    }
	}
	d = (struct DBF *)malloc(sizeof(struct DBF)); /* allocate space for DBF */

	strcpy(d->filename,*argv);

	if(!strchr(d->filename,'.')) 	/* default to .dbf file */
	    strcat(d->filename,".DBF");


	if((errornum = d_open(d)) != 0) /* open file */
	{
	    printf("Error opening file: ");
	    switch (errornum)
	    {
	    case OUT_OF_MEM:
		printf("Not enough memory.\n");
		break;
	    case NO_FILE:
		printf("Can not open file %s.\n",d->filename);
		break;
	    case BAD_FORMAT:
		printf("File %s is not a dBASE III file.\n",d->filename);
		break;
	    }
	    free(d);
	    exit(1);
	}

	stats(d) ;

	puts("\n     List of records\n"
	    "     ~~~~~~~~~~~~~~~");
	for(rec=1; rec<=min(listrecords,d->records); rec++)
	{
	    printf("\n%-5u",rec);

	    d_getrec(d,(long)rec);
	    for(fld=1;fld<=min(listfields,d->num_fields);fld++)
	    {
		if(d_getfld(d,fld,buff) == 'D')
		    fdate(buff);
		printf(" %s",buff);
	    }
	}
	puts("\n\f") ;

	d_close(d);
	free(d);
    }
}

stats(struct DBF *d)
{
    int i ;
    struct FIELD_RECORD *f;

    /* report stats */
    printf("\nFilename:          %s\n",d->filename);
    printf("Header length:     %-u\n",d->header_length);
    printf("Number of fields:  %-u\n",d->num_fields);
    if(d->dbf_version == DB3FILE)
	printf("Version:           dBASE III\n");
    else
	printf("Version:           dBASE III with Memo file\n");
    printf("Updated:           %02u/%02u/%2u\n",
    d->update_mo,d->update_day,d->update_yr);
    printf("Number of records: %-u\n",d->records);
    printf("Record length:     %-u\n",d->record_length);
    f = d->fields_ptr;
    printf("\n\n        FILE STRUCTURE\n");
    printf("\n  # FIELD NAME TYPE  LEN   DEC");
    printf("\n--------------------------------\n");
    for(i=1;i<=d->num_fields;++i,f++)
	printf("%3u %-12s %c  %3u   %3u\n",i,f->name,f->typ,f->len,f->dec);
    return ;
}

