; MOUSE DRIVER FOR:
; mouse systems mouse clone. Replaces Keyboard to allow mouse.
;====================================================================
; this area is for int 16h and int 0ch                              |
; It consists of 5 routines:                                        |
;                                                                   |
;   PROCEDURE INITCOM initializes the serial port vectors for       |
;   the serial port requested. The calling format is the            |
;   same as the ah 0 call of int 16                                 |
;                                                                   |
;   PROCEDURE INT0B is a 20 word circular buffer                    |
;                                                                   |
;   PROCEDURE INT0C same as int0b except for com1                   |
;                                                                   |
;   PROCEDURE INT16 is a replacement for the BIOS routine           |
;                                                                   |
;   PROCEDURE DECOM deinitializes the serial port vectors           |
;                                                                   |
;====================================================================
print  macro string
      push  ds
      push  cs
      pop   ds
      mov   ah,9
      lea   dx,string
      int   21h
      pop   ds
      endm
;
CSEG   SEGMENT PARA PUBLIC 'CODE'
   ASSUME CS:CSEG,DS:CSEG,SS:CSEG,ES:CSEG ;ALREADY SET BY DOS LOADER
      org  100h
Enter: jmp install
;====================================================================
;                  DATA AREAS                                       |
;====================================================================
com1loc  dw  0
com2loc  dw  0
com1on   db  0
com2on   db  0
set16    db  0
int16old equ this dword
int16ip  dw  0
int16cs  dw  0
int09old equ this dword
int09ip  dw  0
int09cs  dw  0
int0cip  dw  0
int0ccs  dw  0
int0bip  dw  0
int0bcs  dw  0
buffsize equ  20
buffer1  dw   buffsize dup(0)
buff1count dw  0
buff1point dw  0
buff1tail  dw  0
buff1full  db  0
com1line   db  0
com1modem  db  0
buffer2  dw   buffsize dup(0)
buff2count dw  0
buff2point dw  0
buff2tail  dw  0
buff2full  db  0
com2line   db  0
com2modem  db  0
semaphore1 db  0
comactive  db  0
func16   db  0
mphase  dw  0
no_install  db  0
;
;====================================================================
;                  PROCEDURES                                       |
translate  proc  near
; translates the al character from the mouse into a keystroke
;
; there is a byte in al, we have to check it to determine
; which keystroke it is
; if the number is from 128 to 135 it is a control
    mov  ah,0
    cmp  ax,128
    jl   no_control
    cmp  ax,135
    jg   no_control
    mov  mphase,0   ; phase 0 to 3 of data from mouse
; it could be a return or an esc
    cmp  ax,131  ;return
    jne  no_131
    mov  ax,1c0dh
    ret
no_131:
    cmp  ax,133
    jne  no_133
    mov  ax,011bh
    ret
no_133:
    cmp  ax,130   ; middle and right make it the / character
    jne  no_130
    mov  ax,352fh
    ret
no_130:
; for now this is enough
     mov   ax,0
   ret
no_control:
; we must have a byte from the mouse
     inc   mphase
     cmp   mphase,4
     jl    ok_phase
; something screwy in the phase, time to escape
     mov   ax,0
     ret
ok_phase:
; check to see if we have to do anything
     cmp   ax,0
     jne   no_zero_phase
     ret
no_zero_phase:
     cmp   mphase,1
     je    phase_1
     cmp   mphase,2
     je    phase_2
; ignore the 3 and 4 phase
ignore_data:
     mov   ax,0
     ret
phase_1:
; left or right
     cmp  ax,127
     jg   phase_left
; right
    cmp   ax,2   ; must be greater than 3
    jle   ignore_data
    mov  ax,4d00h
    ret
phase_left:
    neg  al
    cmp   ax,2   ; must be greater than 3
    jle   ignore_data
    mov  ax,4b00h
    ret
     ret
phase_2:
; up or down
     cmp  ax,127
     jg   phase_down
; up phase
    cmp   ax,2   ; must be greater than 3
    jle   ignore_data
    mov  ax,4800h
    ret
phase_down:
    neg  al
    cmp   ax,2   ; must be greater than 3
    jle   ignore_data
    mov  ax,5000h
     ret
translate  endp
;=================================================================
;
;    INT0C Procedure
;
;=================================================================
;
int0c  proc   near
; this is the circular buffer routine called by a hardware
; interrupt to IRQ4
;
;  save the world
    push  ax
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
; we must interogate the various registers of the 8250
; to grab any information about the interrupt.
    mov   ax,cs
    mov   ds,ax
    mov   es,dx
; first check and see if it was the data register that did the
; interrupt
    mov   dx,com1loc
    inc   dx
    inc   dx  ; int ident reg
    in    al,dx
    sti
    test  al,4 ; was it data?
    jnz   do0cdata
    jmp   do0cstates1
do0cdata:
; we don't know the meanings of the comloc+2 register yet so ignore
    mov  dx,com1loc
    add  dx,5   ; line status register
    in   al,dx
    mov  com1line,al
    test al,1   ; is a byte ready
    jz   do0cstates
; we have a byte ready
    mov  dx,com1loc
    nop
    in   al,dx  ; got the byte
    call  translate
    cmp   ax,0
    je    exit_int0c
; now put it to the circular buffer
; first is the buffer full?
    cli
    cmp  buff1count,buffsize
    jl   putbuff0c
    mov  buff1full,1
    jmp  do0cstates1
putbuff0c:
    mov  di,buff1point
    shl  di,1
    mov  buffer1[di],ax
    inc  buff1point
    inc  buff1count
    cmp  buff1point,buffsize
    jl   putbuff0c1
    mov  buff1point,0
putbuff0c1:
    jmp  exit_int0c
do0cstates1:
; nothing was happening with the data so we should test the states
    mov  dx,com1loc
    in   al,dx
    nop
    nop
    add  dx,5
    in   al,dx
    mov  com1line,al
do0cstates:
    mov  dx,com1loc
    add  dx,6
    nop
    in   al,dx
    mov  com1modem,al
exit_int0c:
; reset the interrupt line
    mov  al,20h
    out  20h,al  ; this sends an End Of Interrupt command
                 ; to the 8259a interrupt controller
    sti
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    pop  ax
    iret
int0c   endp
;====================================================================
;
;    INT0B Procedure
;
;=================================================================
;
int0b  proc   near
; this is the circular buffer routine called by a hardware
; interrupt to IRQ3
;
;  save the world
    push  ax
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
; we must interogate the various registers of the 8250
; to grab any information about the interrupt.
    mov   ax,cs
    mov   ds,ax
    mov   es,dx
;   mov   dx,com2loc
;   add   dx,4
;   mov   al,0bh  ; restore the com lines
;   out   dx,al
; first check and see if it was the data register that did the
; interrupt
    mov   dx,com2loc
    inc   dx
    inc   dx  ; int ident reg
    in    al,dx
    sti
    test  al,4 ; was it data?
    jnz   do0bdata
    jmp   do0bstates1
do0bdata:
; we don't know the meanings of the comloc+2 register yet so ignore
    mov  dx,com2loc
    add  dx,5   ; line status register
    in   al,dx
    mov  com2line,al
    test al,1   ; is a byte ready
    jz   do0bstates
; we have a byte ready
    mov  dx,com2loc
    nop
    in   al,dx  ; got the byte
    call  translate
    cmp   ax,0
    je    exit_int0b
; now put it to the circular buffer
; first is the buffer full?
    cli
    cmp  buff2count,buffsize
    jl   putbuff0b
    mov  buff2full,1
    jmp  do0bstates1
putbuff0b:
    mov  di,buff2point
    shl  di,1
    mov  buffer2[di],ax
    inc  buff2point
    inc  buff2count
    cmp  buff2point,buffsize
    jl   putbuff0b1
    mov  buff2point,0
putbuff0b1:
    jmp  exit_int0b
do0bstates1:
; nothing was happening with the data so we should test the states
    mov  dx,com2loc
    in   al,dx
    nop
    nop
    add  dx,5
    in   al,dx
    mov  com2line,al
do0bstates:
    mov  dx,com2loc
    add  dx,6
    nop
    in   al,dx
    mov  com2modem,al
exit_int0b:
; reset the interrupt line
    mov  al,20h
    out  20h,al  ; this sends an End Of Interrupt command
                 ; to the 8259a interrupt controller
    sti
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    pop  ax
    iret
int0b   endp
;====================================================================
;
;    INT16 Procedure
;
;=================================================================
;
int16  proc   near
;
; this imitates the int16 except that if there is something in the
; mouse buffer that goes first
;
; AH = 0 is get a byte
; AH = 1 is get a byte, but don't bump the buffer count
;
    cmp   ah,0
    je    do16
    cmp   ah,1
    je    do16
    cmp   ax,'KG'
    jne   leave16
    push  cs
    pop   ds
    push  cs
    pop   es
    mov   ax,'EC'
    iret
leave16:
    jmp   cs:int16old
do16:
    cmp  cs:comactive,1
    jne  do_16_1
    cmp  cs:buff2count,0
    jle  leave16
    jmp  save_16
do_16_1:
    cmp  cs:buff1count,0
    jle  leave16
save_16:
    mov  cs:func16,ah
;  save the world
    sti
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
    mov   bx,cs
    mov   es,bx
    mov   ds,bx
    jmp   do16get
exit_16h:
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
; have to return with the zero flag set if ah,0
    cmp  ax,0  ; zero means no char waiting
; retf 2
    db 0cah,02h,00h
    iret
leave_16h:
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    jmp  leave16
do16get:
; this does a get from the cirular buffer
    cmp  comactive,1
    jne  do16get1
    jmp  do16get2
do16get1:
    cli
    mov  ah,0
    mov  si,buff1tail
    shl  si,1
    mov  ax,buffer1[si]
    cmp  func16,0
    jne  no_inc_si1
    mov  si,buff1tail
    inc  si
    cmp  si,buffsize
    jl   g16c12
    mov  si,0
g16c12:
    mov  buff1tail,si
    dec  buff1count
no_inc_si1:
    sti
    jmp  exit_16h
do16get2:
    cli
got16char2:
    mov  si,buff2tail
    shl  si,1
    mov  ax,buffer2[si]
    cmp  func16,0
    jne  no_inc_si2
    mov  si,buff2tail
    inc  si
    cmp  si,buffsize
    jl   g16c22
    mov  si,0
g16c22:
    mov  buff2tail,si
    dec  buff2count
no_inc_si2:
    sti
    jmp  exit_16h
int16  endp
;====================================================================
;
;   Reset irq4
;
;====================================================================
irq4  proc  near
; set irq4 to be com1
     sti
     in   al,21h   ; get the interrup mask
     and  al,0efh  ; add ours to it
     out  21h,al
     mov  dx,com1loc
     add  dx,4
     mov  al,0bh   ; set dtr,rts and out2 on
     out  dx,al
     dec  dx
     dec  dx       ; set up for interrupt id register
     nop
     nop
     in   al,dx
     cmp  al,1     ; did it work?
     je   dc1_2
     mov  dx,com1loc
     in   al,dx    ; flush the input register
dc1_2:
     mov  dx,com1loc
     inc  dx
     mov  al,01h  ; 1 means data interrupts, F means all?
     out  dx,al  ; enable receive data interrupts only
; clear the interrupts
     mov  al,64h
     out  20h,al
; now rq4 pulls at int0c
      ret
irq4  endp
;
;====================================================================
;
; set up irq3 for com2
;
;====================================================================
irq3 proc  near
; set irq3 to be com2
     sti
     in   al,21h   ; get the interrup mask
     and  al,0f7h  ; add ours to it
     out  21h,al
     mov  dx,com2loc
     add  dx,4
     mov  al,0bh   ; set dtr,rts and out2 on
     out  dx,al
     dec  dx
     dec  dx       ; set up for interrupt id register
     in   al,dx
     cmp  al,1     ; did it work?
     je   dc2_2
     mov  dx,com2loc
     in   al,dx    ; flush the input register
dc2_2:
     mov  dx,com2loc
     inc  dx
     mov  al,0fh
     out  dx,al  ; enable receive data interrupts only
; clear the interrupts
     mov  al,64h
     out  20h,al
; now rq3 pulls at int0b
     ret
irq3 endp
;====================================================================
;
;  Install the thing as resident
;
;
;====================================================================
install  proc  near
; check the command line
    mov   cl,ds:[80h]
    mov   ch,0
    mov   di,81h
    cmp   cx,0
    jg    cont_cx
bad_command:
    print mess1
    int   20h
cont_cx:
    cld
    mov   al,'1'
    push  di
    push  cx
repnz scasb
    je    got_port
    pop  cx
    pop  di
    mov  al,'2'
    push  di
    push  cx
repnz  scasb
    je    got_port
    pop  cx
    pop  di
    jmp  bad_command
got_port:
    and  ax,000fh
    dec  ax
; check to see if we are already installed
    push  ax
    mov   ax,'KG'
    int   16h
    cmp   ax,"EC"
    jne   ok_to_install
    mov   no_install,1
    print mess2
ok_to_install:
; install the fool thing
    pop   dx  ; com port in dx
    mov   ah,0
    mov   bx,0
    mov   cx,0
    mov   al,83h
    call  initcom
    cmp   no_install,1
    je    dont_install
    print mess4
    lea   dx,install
    int   27h
dont_install:
    print mess3
    push  cs
    pop   ds
    int   20h
install  endp
mess1   db   7,'The command to Install the OPTIKS Mouse driver',13,10
        db   'for the Mouse Systems Mouse is:',13,10
        db   'OKMSMI n',13,10
        db   'where n is the serial port being used by the mouse',13,10,'$'
mess2   db   'The Mouse driver has been previously installed',13,10
        db   'Mouse options are being reinitialized',13,10,'$'
mess3   db   'Mouse options reinitialized',13,10,'$'
mess4   db   'OPTIKS Mouse driver for the Mouse Systems Mouse',13,10
        db   'is now installed.',13,10
        db   'Copyright 1987 by Keith P. Graham',13,10,'$'
;====================================================================
;
;    INITCOM Procedure
;
;=================================================================
;
initcom  proc  near
; first get the com1 io address
; calling sequence is same as for INT 14H
; Baud etc is determined by the caller
; COM line is in BX , 0 = 1 , 1 = 2
     push  ax
     push  cx
     push  dx
     push  ax
     mov   ax,40h
     push  ds
     mov   ds,ax
     mov   ax,word ptr ds:[0]
     mov   bx,word ptr ds:[2]
     pop   ds
     mov   com1loc,ax
     mov   com2loc,bx
; now initialize the com port
     pop   ax
     mov   ah,0    ; initializes the com port params in al?
     push  dx
     int   14h     ;serial i/o
; now establish a new int16h
     cmp   set16,1
     je    no_set16
     push  es
     mov   ah,35h
     mov   al,16h
     int   21h     ; gets the interrupt
     mov   int16ip,bx
     mov   int16cs,es
     pop   es
     mov   set16,1
; now have 16 point to int16 procedure
     mov   ah,25h
     mov   al,16h
     lea   dx,int16
     int   21h
no_set16:
; check to see if the irqline needs setting
     pop   bx
     push  bx
     cmp   com1on[bx],1
     jne   cdone_init
     jmp   cont_cominit
cdone_init:
; must set the irq line to either a or b
     cmp   bx,0
     je    do_com1
     jmp   do_com2
do_com1:
     mov  comactive,0
; set interrupt to point to int0c
     push  es
     mov   ah,35h
     mov   al,0ch
     int   21h     ; gets the interrupt
     mov   int0cip,bx
     mov   int0ccs,es
     pop   es
; now have 0c point to int0c procedure
     mov   ah,25h
     mov   al,0ch
     lea   dx,int0c
     int   21h
; now turn on irq4
     jmp  cont_cominit
do_com2:
     mov  comactive,1
; set interrupt to point to int0c
     push  es
     mov   ah,35h
     mov   al,0bh
     int   21h     ; gets the interrupt
     mov   int0bip,bx
     mov   int0bcs,es
     pop   es
; now have 0b point to int0b procedure
     mov   ah,25h
     mov   al,0bh
     lea   dx,int0b
     int   21h
; set up irq3
; now establish that the com port is in control
cont_cominit:
     pop   bx
     push  bx
     cmp   bx,0
     je    set_com1
     jmp   set_com2
set_com1:
     call  irq4
     jmp   done_set
set_com2:
     call irq3
done_set:
     pop    bx
     mov   com1on[bx],1
; what else must be done?
done_init:
     pop   dx
     pop   cx
     pop   ax
     ret
initcom  endp
CSEG   ENDS
   END  enter
